# Generated by Django 3.1.3 on 2021-01-07 23:09

# import uuid

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0017_auto_20201203_2107"),
    ]

    operations = [
        migrations.AddField(
            model_name="Address",
            name="uuid",
            field=models.UUIDField(
                editable=True, unique=False, verbose_name="UUID", null=True
            ),
        ),
        migrations.AddField(
            model_name="CustomerProfile",
            name="uuid",
            field=models.UUIDField(
                editable=True, unique=False, verbose_name="UUID", null=True
            ),
        ),
    ]
