# Generated by Django 5.2.9 on 2025-12-22 17:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0044_site_settings_stripe_processor_rename"),
    ]

    operations = [
        migrations.AlterField(
            model_name="address",
            name="country",
            field=models.IntegerField(
                choices=[
                    (20, "Andorra"),
                    (784, "United Arab Emirates"),
                    (4, "Afghanistan"),
                    (28, "Antigua And Barbuda"),
                    (660, "Anguilla"),
                    (8, "Albania"),
                    (51, "Armenia"),
                    (530, "Netherlands Antilles"),
                    (24, "Angola"),
                    (10, "Antarctica"),
                    (32, "Argentina"),
                    (16, "American Samoa"),
                    (40, "Austria"),
                    (36, "Australia"),
                    (533, "Aruba"),
                    (31, "Azerbaijan"),
                    (70, "Bosnia And Herzegowina"),
                    (52, "Barbados"),
                    (50, "Bangladesh"),
                    (56, "Belgium"),
                    (854, "Burkina Faso"),
                    (100, "Bulgaria"),
                    (48, "Bahrain"),
                    (108, "Burundi"),
                    (204, "Benin"),
                    (60, "Bermuda"),
                    (96, "Brunei Darussalam"),
                    (68, "Bolivia"),
                    (76, "Brazil"),
                    (44, "Bahamas"),
                    (64, "Bhutan"),
                    (74, "Bouvet Island"),
                    (72, "Botswana"),
                    (112, "Belarus"),
                    (84, "Belize"),
                    (124, "Canada"),
                    (166, "Cocos (Keeling) Islands"),
                    (180, "The Democratic Republic Of The Congo"),
                    (140, "Central African Republic"),
                    (178, "Congo"),
                    (756, "Switzerland"),
                    (384, "Cote D'ivoire"),
                    (184, "Cook Islands"),
                    (152, "Chile"),
                    (120, "Cameroon"),
                    (156, "China"),
                    (170, "Colombia"),
                    (188, "Costa Rica"),
                    (192, "Cuba"),
                    (132, "Cape Verde"),
                    (162, "Christmas Island"),
                    (196, "Cyprus"),
                    (203, "Czech Republic"),
                    (276, "Germany"),
                    (262, "Djibouti"),
                    (840, "United States"),
                    (392, "Japan"),
                    (484, "Mexico"),
                ],
                default=840,
                verbose_name="Country/Region",
            ),
        ),
    ]
