# Generated by Django 3.1 on 2020-09-30 20:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0004_offer_offer_description"),
    ]

    operations = [
        migrations.AddField(
            model_name="address",
            name="first_name",
            field=models.CharField(
                blank=True, max_length=150, verbose_name="First Name"
            ),
        ),
        migrations.AddField(
            model_name="address",
            name="last_name",
            field=models.CharField(
                blank=True, max_length=150, verbose_name="Last Name"
            ),
        ),
    ]
