# Generated by Django 3.1.3 on 2021-09-13 20:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0023_profile_null_false"),
    ]

    operations = [
        migrations.AddField(
            model_name="offer",
            name="deleted",
            field=models.BooleanField(default=False, verbose_name="Deleted"),
        ),
    ]
