# Generated by Django 3.2.13 on 2022-06-07 17:12

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0034_subscription"),
    ]

    operations = [
        migrations.AddField(
            model_name="payment",
            name="subscription",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="payments",
                to="vendor.subscription",
                verbose_name="Subscription",
            ),
        ),
        migrations.AddField(
            model_name="receipt",
            name="subscription",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="receipts",
                to="vendor.subscription",
                verbose_name="Subscription",
            ),
        ),
    ]
