# Generated by Django 3.2.13 on 2022-06-07 16:49

import uuid

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0033_payment_status"),
    ]

    operations = [
        migrations.CreateModel(
            name="Subscription",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        auto_now_add=True, verbose_name="date created"
                    ),
                ),
                (
                    "updated",
                    models.DateTimeField(auto_now=True, verbose_name="last updated"),
                ),
                ("deleted", models.BooleanField(default=False, verbose_name="Deleted")),
                (
                    "uuid",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        unique=True,
                        verbose_name="UUID",
                    ),
                ),
                (
                    "gateway_id",
                    models.CharField(
                        max_length=80, verbose_name="Subscription Gateway ID"
                    ),
                ),
                (
                    "auto_renew",
                    models.BooleanField(default=False, verbose_name="Auto Renew"),
                ),
                (
                    "meta",
                    models.JSONField(
                        blank=True, default=dict, null=True, verbose_name="Meta"
                    ),
                ),
                (
                    "profile",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="subscriptions",
                        to="vendor.customerprofile",
                        verbose_name="Purchase Profile",
                    ),
                ),
                (
                    "status",
                    models.IntegerField(
                        choices=[
                            (10, "Pause"),
                            (20, "Active"),
                            (30, "Canceled"),
                            (40, "Suspended"),
                        ],
                        default=0,
                        verbose_name="Status",
                    ),
                ),
            ],
            options={
                "verbose_name": "Subscription",
                "verbose_name_plural": "Subscriptions",
            },
        ),
    ]
