# Generated by Django 3.1.3 on 2020-11-12 01:38

import django.db.models.deletion
from django.db import migrations, models

import vendor.models.utils


class Migration(migrations.Migration):

    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        ("vendor", "0009_auto_20201111_0743"),
    ]

    operations = [
        migrations.AlterField(
            model_name="customerprofile",
            name="site",
            field=models.ForeignKey(
                default=vendor.models.utils.set_default_site_id,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="customer_profile",
                to="sites.site",
                verbose_name="Site",
            ),
        ),
        migrations.AlterField(
            model_name="invoice",
            name="site",
            field=models.ForeignKey(
                default=vendor.models.utils.set_default_site_id,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="invoices",
                to="sites.site",
                verbose_name="Site",
            ),
        ),
        migrations.AlterField(
            model_name="offer",
            name="site",
            field=models.ForeignKey(
                default=vendor.models.utils.set_default_site_id,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="product_offers",
                to="sites.site",
                verbose_name="Site",
            ),
        ),
        migrations.AlterField(
            model_name="payment",
            name="billing_address",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="vendor.address",
                verbose_name="Billing Address",
            ),
        ),
        migrations.AlterField(
            model_name="payment",
            name="created",
            field=models.DateTimeField(auto_now_add=True, verbose_name="Date Created"),
        ),
        migrations.AlterField(
            model_name="price",
            name="cost",
            field=models.FloatField(blank=True, null=True, verbose_name="Cost"),
        ),
        migrations.AlterField(
            model_name="price",
            name="offer",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="prices",
                to="vendor.offer",
                verbose_name="Offer",
            ),
        ),
        migrations.AlterField(
            model_name="receipt",
            name="meta",
            field=models.JSONField(default=dict, verbose_name="Meta"),
        ),
        migrations.AlterField(
            model_name="taxclassifier",
            name="taxable",
            field=models.BooleanField(verbose_name="Taxable"),
        ),
    ]
