# Generated by Django 3.1.3 on 2021-01-07 23:10

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0019_fill_uuid_address_profile"),
    ]

    operations = [
        migrations.AlterField(
            model_name="Address",
            name="uuid",
            field=models.UUIDField(
                default=uuid.uuid4, editable=False, unique=True, verbose_name="UUID"
            ),
        ),
        migrations.AlterField(
            model_name="CustomerProfile",
            name="uuid",
            field=models.UUIDField(
                default=uuid.uuid4, editable=False, unique=True, verbose_name="UUID"
            ),
        ),
    ]
