# Generated by Django 3.1.3 on 2020-12-03 21:07

import django.contrib.sites.managers
import django.db.models.manager
from django.db import migrations

import vendor.models.offer


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0016_auto_20201203_2011"),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="offer",
            managers=[
                ("objects", django.db.models.manager.Manager()),
                ("on_site", django.contrib.sites.managers.CurrentSiteManager()),
                ("on_site_active", vendor.models.offer.ActiveCurrentSiteManager()),
            ],
        ),
    ]
