# Generated by Django 3.1.3 on 2021-04-08 23:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0020_lock_uuid_address_profile"),
    ]

    operations = [
        migrations.AlterField(
            model_name="address",
            name="country",
            field=models.IntegerField(
                choices=[
                    (20, "Andorra"),
                    (784, "United Arab Emirates"),
                    (4, "Afghanistan"),
                    (28, "Antigua And Barbuda"),
                    (660, "Anguilla"),
                    (8, "Albania"),
                    (51, "Armenia"),
                    (530, "Netherlands Antilles"),
                    (24, "Angola"),
                    (10, "Antarctica"),
                    (32, "Argentina"),
                    (16, "American Samoa"),
                    (40, "Austria"),
                    (36, "Australia"),
                    (533, "Aruba"),
                    (31, "Azerbaijan"),
                    (70, "Bosnia And Herzegowina"),
                    (52, "Barbados"),
                    (50, "Bangladesh"),
                    (56, "Belgium"),
                    (854, "Burkina Faso"),
                    (100, "Bulgaria"),
                    (48, "Bahrain"),
                    (108, "Burundi"),
                    (204, "Benin"),
                    (60, "Bermuda"),
                    (96, "Brunei Darussalam"),
                    (68, "Bolivia"),
                    (76, "Brazil"),
                    (44, "Bahamas"),
                    (64, "Bhutan"),
                    (74, "Bouvet Island"),
                    (72, "Botswana"),
                    (112, "Belarus"),
                    (84, "Belize"),
                    (124, "Canada"),
                    (166, "Cocos (Keeling) Islands"),
                    (180, "The Democratic Republic Of The Congo"),
                    (140, "Central African Republic"),
                    (178, "Congo"),
                    (756, "Switzerland"),
                    (384, "Cote D'ivoire"),
                    (184, "Cook Islands"),
                    (152, "Chile"),
                    (120, "Cameroon"),
                    (156, "China"),
                    (170, "Colombia"),
                    (188, "Costa Rica"),
                    (192, "Cuba"),
                    (132, "Cape Verde"),
                    (162, "Christmas Island"),
                    (196, "Cyprus"),
                    (203, "Czech Republic"),
                    (276, "Germany"),
                    (262, "Djibouti"),
                    (208, "Denmark"),
                    (212, "Dominica"),
                    (214, "Dominican Republic"),
                    (12, "Algeria"),
                    (218, "Ecuador"),
                    (233, "Estonia"),
                    (818, "Egypt"),
                    (732, "Western Sahara"),
                    (232, "Eritrea"),
                    (724, "Spain"),
                    (231, "Ethiopia"),
                    (246, "Finland"),
                    (242, "Fiji"),
                    (238, "Falkland Islands (Malvinas)"),
                    (583, "Federated States Of Micronesia"),
                    (234, "Faroe Islands"),
                    (250, "France"),
                    (249, "France, Metropolitan"),
                    (266, "Gabon"),
                    (826, "United Kingdom"),
                    (308, "Grenada"),
                    (268, "Georgia"),
                    (254, "French Guiana"),
                    (288, "Ghana"),
                    (292, "Gibraltar"),
                    (304, "Greenland"),
                    (270, "Gambia"),
                    (324, "Guinea"),
                    (312, "Guadeloupe"),
                    (226, "Equatorial Guinea"),
                    (300, "Greece"),
                    (239, "South Georgia And The South Sandwich Islands"),
                    (320, "Guatemala"),
                    (316, "Guam"),
                    (624, "Guinea-bissau"),
                    (328, "Guyana"),
                    (344, "Hong Kong"),
                    (334, "Heard And Mc Donald Islands"),
                    (340, "Honduras"),
                    (191, "Croatia (Local Name: Hrvatska)"),
                    (332, "Haiti"),
                    (348, "Hungary"),
                    (360, "Indonesia"),
                    (372, "Ireland"),
                    (376, "Israel"),
                    (356, "India"),
                    (86, "British Indian Ocean Territory"),
                    (368, "Iraq"),
                    (364, "Iran (Islamic Republic Of Iran)"),
                    (352, "Iceland"),
                    (380, "Italy"),
                    (388, "Jamaica"),
                    (400, "Jordan"),
                    (392, "Japan"),
                    (404, "Kenya"),
                    (417, "Kyrgyzstan"),
                    (116, "Cambodia"),
                    (296, "Kiribati"),
                    (174, "Comoros"),
                    (659, "Saint Kitts And Nevis"),
                    (408, "Democratic People's Republic Of Korea"),
                    (410, "Republic Of Korea"),
                    (414, "Kuwait"),
                    (136, "Cayman Islands"),
                    (398, "Kazakhstan"),
                    (418, "Lao People's Democratic Republic"),
                    (422, "Lebanon"),
                    (662, "Saint Lucia"),
                    (438, "Liechtenstein"),
                    (144, "Sri Lanka"),
                    (430, "Liberia"),
                    (426, "Lesotho"),
                    (440, "Lithuania"),
                    (442, "Luxembourg"),
                    (428, "Latvia"),
                    (434, "Libyan Arab Jamahiriya"),
                    (504, "Morocco"),
                    (492, "Monaco"),
                    (498, "Republic Of Moldova"),
                    (450, "Madagascar"),
                    (584, "Marshall Islands"),
                    (807, "The Former Yugoslav Republic Of Macedonia "),
                    (466, "Mali"),
                    (104, "Myanmar"),
                    (496, "Mongolia"),
                    (446, "Macau"),
                    (580, "Northern Mariana Islands"),
                    (474, "Martinique"),
                    (478, "Mauritania"),
                    (500, "Montserrat"),
                    (470, "Malta"),
                    (480, "Mauritius"),
                    (462, "Maldives"),
                    (454, "Malawi"),
                    (484, "Mexico"),
                    (458, "Malaysia"),
                    (508, "Mozambique"),
                    (516, "Namibia"),
                    (540, "New Caledonia"),
                    (562, "Niger"),
                    (574, "Norfolk Island"),
                    (566, "Nigeria"),
                    (558, "Nicaragua"),
                    (528, "Netherlands"),
                    (578, "Norway"),
                    (524, "Nepal"),
                    (520, "Nauru"),
                    (570, "Niue"),
                    (554, "New Zealand"),
                    (512, "Oman"),
                    (591, "Panama"),
                    (604, "Peru"),
                    (258, "French Polynesia"),
                    (598, "Papua New Guinea"),
                    (608, "Philippines"),
                    (586, "Pakistan"),
                    (616, "Poland"),
                    (666, "St. Pierre And Miquelon"),
                    (612, "Pitcairn"),
                    (630, "Puerto Rico"),
                    (620, "Portugal"),
                    (585, "Palau"),
                    (600, "Paraguay"),
                    (634, "Qatar"),
                    (638, "Reunion"),
                    (642, "Romania"),
                    (643, "Russian Federation"),
                    (646, "Rwanda"),
                    (682, "Saudi Arabia"),
                    (90, "Solomon Islands"),
                    (690, "Seychelles"),
                    (729, "Sudan"),
                    (752, "Sweden"),
                    (702, "Singapore"),
                    (654, "St. Helena"),
                    (705, "Slovenia"),
                    (744, "Svalbard And Jan Mayen Islands"),
                    (703, "Slovakia (Slovak Republic)"),
                    (694, "Sierra Leone"),
                    (674, "San Marino"),
                    (686, "Senegal"),
                    (706, "Somalia"),
                    (740, "Suriname"),
                    (678, "Sao Tome And Principe"),
                    (222, "El Salvador"),
                    (760, "Syrian Arab Republic"),
                    (748, "Swaziland"),
                    (796, "Turks And Caicos Islands"),
                    (148, "Chad"),
                    (260, "French Southern Territories"),
                    (768, "Togo"),
                    (764, "Thailand"),
                    (762, "Tajikistan"),
                    (772, "Tokelau"),
                    (795, "Turkmenistan"),
                    (788, "Tunisia"),
                    (776, "Tonga"),
                    (626, "East Timor"),
                    (792, "Turkey"),
                    (780, "Trinidad And Tobago"),
                    (798, "Tuvalu"),
                    (158, "Taiwan, Province Of China"),
                    (834, "United Republic Of Tanzania"),
                    (804, "Ukraine"),
                    (800, "Uganda"),
                    (581, "United States Minor Outlying Islands"),
                    (840, "United States"),
                    (858, "Uruguay"),
                    (860, "Uzbekistan"),
                    (336, "Holy See (Vatican City State)"),
                    (670, "Saint Vincent And The Grenadines"),
                    (862, "Venezuela"),
                    (92, "Virgin Islands (British)"),
                    (850, "Virgin Islands (U.S.)"),
                    (704, "Viet Nam"),
                    (548, "Vanuatu"),
                    (876, "Wallis And Futuna Islands"),
                    (882, "Samoa"),
                    (887, "Yemen"),
                    (175, "Mayotte"),
                    (710, "South Africa"),
                    (894, "Zambia"),
                    (716, "Zimbabwe"),
                ],
                default=840,
                verbose_name="Country/Region",
            ),
        ),
    ]
